<?php

/**
 * PHP implementation of Sprockets.
 *
 * Inspirated by:
 * http://getsprockets.org/
 * https://github.com/stuartloxton/php-sprockets
 *
 * TODO:
 *   - missing directive implementations
 *   - other comment type implementations
 *
 * @author Gabor Sar <gabor.sar@artoon.hu>
 */
class Sprockets
{
    /**
     * Cache manager object.
     *
     * @var \Sprockets\Cache\CacheManagerInterface
     */
    protected $_cacheManager = null;

    /**
     * Context object.
     *
     * @var \Sprockets\Context\ContextInterface
     */
    protected $_context = null;

    /**
     * Loadpath manager object.
     *
     * @var \Sprockets\Loadpath\LoadpathManagerInterface
     */
    protected $_loadpathManager = null;

    /**
     * List of processors.
     *
     * @var array
     */
    protected $_processors = array();

    /**
     * Constructor method. Sets context and loadpath manger objects.
     */
    public function __construct()
    {
        // set context object
        $this->_context = new \Sprockets\Context\Context();

        // set loadpath manager object
        $this->_loadpathManager = new \Sprockets\Loadpath\LoadpathManager();
    }

    /**
     * Handles a request.
     * Processes the first file (and all the files...).
     *
     * @param string $filename
     * @throws Exception
     * @return string
     */
    public function handleRequest($filename)
    {
        // check whether the given parameter ($filename) is a stirng,
        // throw an exception otherwise
        if (!is_string($filename)) {
            throw new Exception('Filename must be a string', 500);
        }

        // correct filename
        $filename = realpath($this->_context->getBasepath() . '/' . $filename);

        // process the first file (and all the files as well)
        $content = $this->_processFile($filename);

        // store content
        $this->_context->setContent($content);

        // execute all processor
        $this->_executeAllProcessors();

        // retrun content
        return $this->_context->getContent();
    }

    /**
     * Processes one file.
     *
     * @param string $filename Contains absolute path.
     * @return string|boolean Content of file after processing.
     */
    protected function _processFile($filename)
    {
        // check whether filename is valid
        if (!$this->_isFileValid($filename)) {
            return false;
        }

        // get cache managers
        $cacheManager = $this->_getCacheManager();

        // check whether the file is cached
        if ($cacheManager->exists($filename)) {

            // return cached version instead of rebuliding
            // the whole structure
            return $cacheManager->read($filename);
        }

        // get content of the file
        $content = file_get_contents($filename);

        // get directives (based on the content of the current file)
        $directives = \Sprockets\Directive\DirectiveProcessor::getDirectives($content);

        // process all directives, one by one
        foreach ($directives as $directive) {

            // calculate real path of this directive
            $path = $directive->getPath($this->_context->getExtension());

            // find location of this path
            // can be: realtive to the current file,
            // or relative to one of our loadPaths
            $realpath = $directive->isRelativeToTheCurrentFile()
                ? dirname($filename) . $path
                : $this->_loadpathManager->getPath(
                    $path,
                    $this->_context->getBasepath()
                );

            // get content of this directive
            $scontent = $this->_callAction($directive->getName(), $realpath);

            // replace comment if have to
            if ($scontent !== false) {
                $content = str_replace(
                    $directive->getComment(),
                    $scontent,
                    $content
                );
            }
        }

        // cache the current file
        $cacheManager->write($filename, $content);

        // return content
        return $content . PHP_EOL;
    }

    /**
     * Checks whether a filename is valid.
     * A valid filename have to link to an existing, local file,
     * and must not be excluded!
     * Because of security reasons, it also must be
     * inside our base directory!
     *
     * @param string $filename
     * @return boolean
     */
    protected function _isFileValid($filename)
    {
        // check if it is inside basepath
        $isInsideBasepath = (0 === strpos(realpath($filename),
            realpath($this->_context->getBasepath())));

        // generate result
        // all of our requirements must be succeed !!!
        $isValid = is_file($filename)
            && $isInsideBasepath
            && !$this->_context->isExcluded($filename);

        // return validity
        return $isValid;
    }



    /* *********************** */
    /*   SETTERS AND GETTERS   */
    /* *********************** */



    /**
     * Sets cache maganer.
     *
     * @param \Sprockets\Cache\CacheManagerInterface $cache
     * @return Sprockets
     */
    public function setCacheManager(
        \Sprockets\Cache\CacheManagerInterface $cacheManager
    ) {
        // set cache manager
        $this->_cacheManager = $cacheManager;

        // return self
        return $this;
    }

    /**
     * Returns cache manager.
     *
     * @return \Sprockets\Cache\CacheManagerInterface
     */
    protected function _getCacheManager()
    {
        // use defult cache manager if have to
        if (is_null($this->_cacheManager)) {
            $cacheManager = new \Sprockets\Cache\CacheManager();
            $this->setCacheManager($cacheManager);
        }

        // return cache manager
        return $this->_cacheManager;
    }

    /**
     * Sets basepath.
     *
     * @param string $basepath
     * @throws Exception
     * @return Sprockets
     */
    public function setBasepath($basepath)
    {
        // check whether the given parameter ($basepath) is a string,
        // throw an exception otherwise
        if (!is_string($basepath)) {
            throw new Exception('Basepath must be a string', 500);
        }

        // set basepath
        $this->_context->setBasepath($basepath);

        // return self
        return $this;
    }

    /**
     * Sets extension.
     *
     * @param stirng $extension
     * @throws Exception
     * @return Sprockets
     */
    public function setExtension($extension)
    {
        // check whether the given parameter ($extension) is a stirng,
        // throw an exception otherwise
        if (!is_string($extension)) {
            throw new Exception('Extension must be a string', 500);
        }

        // set extension
        $this->_context->setExtension($extension);

        // return self
        return $this;
    }

    /**
     * Adds a path to the loadPaths (preferd paths) list.
     *
     * @param string $path
     * @throws Exception
     * @return Sprockets
     */
    public function appendPath($path)
    {
        // check whether the given parameter ($path) is a string,
        // throw an exception otherwise
        if (!is_string($path)) {
            throw new Exception('Path must be a string', 500);
        }

        // append path
        $this->_loadpathManager->appendPath($path);

        // retrun self
        return $this;
    }

    /**
     * Stores an extra processor.
     *
     * @param \Sprockets\Processor\AbstractProcessor $processor
     * @return Sprockets
     */
    public function addProcessor(
        \Sprockets\Processor\AbstractProcessor $processor
    ) {
        // give access to context
        $processor->setContext($this->_context);

        // store processor
        $this->_processors[] = $processor;

        // return self
        return $this;
    }



    /* *********** */
    /*   ACTIONS   */
    /* *********** */



    /**
     * Executes all processors.
     */
    protected function _executeAllProcessors()
    {
        foreach ($this->_processors as $processor) {
            $processor->execute();
        }
    }

    /**
     * Calls an action.
     *
     * @param string $name Directive name.
     * @param string $path Parameter of actions.
     * @return string Content of directive after processing.
     */
    protected function _callAction($name, $path)
    {
        // default result = remove comment
        $result = '';

        // select action depending on directive name
        switch ($name) {

            case 'require':
                $result = $this->_require($path);
                break;

            case 'include':
                $result = $this->_include($path);
                break;

            case 'require_directory':
                $result = $this->_require_directory($path);
                break;

            case 'require_tree':
                $result = $this->_require_directory($path, true);
                break;

            case 'require_self':
                $result = 'NOT IMPLEMENTED YET (require_self)';
                break;

            case 'depend_on':
                $result = 'NOT IMPLEMENTED YET (depend_on)';
                break;

            case 'stub':
                $result = $this->_stub($path);
                break;
        }

        return $result;
    }

    /**
     * Requires a file, and makes sure it appears only once.
     *
     * @param string $filename
     * @return string Content of file after processing.
     */
    protected function _require($filename)
    {
        // check whether already reqired
        if (!$this->_context->isRequired($filename)) {

            // mark as required
            $this->_context->setRequired($filename);

            // process file (processes it's directives as well (recursion))
            return $this->_processFile($filename);

        } else {

            // already required => replace comment with an empty string
            return '';
        }
    }

    /**
     * Includes a file.
     *
     * @param string $filename
     * @return string Content of file after processing.
     */
    protected function _include($filename)
    {
        // check whether already required
        if (!$this->_context->isRequired($filename)) {

            // mark as required
            $this->_context->setRequired($filename);
        }

        // process file (processes it's directives as well (recursion))
        return $this->_processFile($filename);
    }

    /**
     * Excludes a file (file cannot appear at all).
     *
     * @param string $filename
     * @return string Content of file after processing.
     */
    protected function _stub($filename)
    {
        // check whether already marked as excluded
        if (!$this->_context->isExcluded($filename)) {

            // mark as excluded
            // if the file already required / included before,
            // this cannot help too much...
            $this->_excluded[] = $filename;
            $this->_context->setExcluded($filename);
        }

        // replace comment with an empty string
        return '';
    }

    /**
     * Requires a directory.
     *
     * @param string $dir
     * @param bool $tree If this is true, than have to require recursively.
     * @return string Content of directory after processing.
     */
    protected function _require_directory($dir, $tree = false)
    {
        // content of all maching file in this directory
        $result = '';

        // get all element of the directory
        $elements = scandir($dir);

        // process all elements one by one,
        // except . and ..
        foreach ($elements as $element) {
            if ($element !== '.' && $element !== '..') {

                // get required infomration about this element
                $path = realpath($dir . '/' . $element);
                $isDir = is_dir($path);
                $isFile = is_file($path);
                $extension = pathinfo($path, PATHINFO_EXTENSION);

                // if this file have the extension we need
                if ($isFile && $extension === $this->_extension) {

                    // require file
                    $result .= $this->_require($path);
                } elseif ($isDir && $tree) {

                    // require directory (recursively)
                    $result .= $this->_require_directory($path, true);
                }
            }
        }

        // return result - content of all matching files in this directory
        return $result;
    }
}
