<?php

namespace Sprockets\Context;

/**
 * Context class.
 * An instantiation of this will contain the context of
 * the whole application.
 *
 * @author Gabor Sar <gabor.sar@artoon.hu>
 * @package Sprockets\Context
 */
class Context
    implements \Sprockets\Context\ContextInterface
{
    /**
     * This is the default location of files,
     * this is the first place where Sprockets
     * will look for them.
     *
     * @var string
     */
    protected $_basepath = '';

    /**
     * Extension of processed files.
     * Default value: js (JavaScript)
     *
     * @var string
     */
    protected $_extension = 'js';

    /**
     * List of required paths.
     *
     * @var array
     */
    protected $_required = array();

    /**
     * List of excluded paths.
     *
     * @var array
     */
    protected $_excluded = array();

    /**
     * Content.
     *
     * @var sring
     */
    protected $_content = '';

    /**
     * Sets basepath.
     *
     * @param string $path
     * @throws Exception
     * @return \Sprockets\Context\Context
     */
    public function setBasepath($path)
    {
        // check whether the given parameter ($path) is a string,
        // throw an exception otherwise
        if (!is_string($path)) {
            throw new Exception('Path must be a string', 500);
        }

        // set basepath
        $this->_basepath = $path;

        // return self
        return $this;
    }

    /**
     * Returns basepath.
     *
     * @return string
     */
    public function getBasepath()
    {
        // return basepath
        return $this->_basepath;
    }

    /**
     * Sets (file)extension.
     *
     * @param string $extension
     * @throws Exception
     * @return \Sprockets\Context\Context
     */
    public function setExtension($extension)
    {
        // check whether the given parameter ($extension) is a string,
        // throw an exception otherwise
        if (!is_string($extension)) {
            throw new Exception('Extension must be a string', 500);
        }

        // set extension
        $this->_extension = $extension;

        // return self
        return $this;
    }

    /**
     * Returns (file)extension.
     *
     * @return string
     */
    public function getExtension()
    {
        // return extension
        return $this->_extension;
    }

    /**
     * Sets a path as required.
     *
     * @param string $path
     * @throws Exception
     * @return \Sprockets\Context\Context
     */
    public function setRequired($path)
    {
        // check whether the given parameter ($path) is a string,
        // throw an exception otherwise
        if (!is_string($path)) {
            throw new Exception('Path must be a string', 500);
        }

        // set path as required
        $this->_required[] = $path;

        // return self
        return $this;
    }

    /**
     * Returns whether a path is already required.
     *
     * @param string $path
     * @throws Exception
     * @return boolean
     */
    public function isRequired($path)
    {
        // check whether the given parameter ($path) is a string,
        // throw an exception otherwise
        if (!is_string($path)) {
            throw new Exception('Path must be a string', 500);
        }

        // return whether path is already required
        return in_array($path, $this->_required);
    }

    /**
     * Sets a path as excluded.
     *
     * @param string $path
     * @throws Exception
     * @return \Sprockets\Context\Context
     */
    public function setExcluded($path)
    {
        // check whether the given parameter ($path) is a string,
        // throw an exception otherwise
        if (!is_string($path)) {
            throw new Exception('Path must be a string', 500);
        }

        // set path as excluded
        $this->_excluded[] = $path;

        // return self
        return $this;
    }

    /**
     * Returns whether a path is excluded.
     *
     * @param string $path
     * @throws Exception
     * @return boolean
     */
    public function isExcluded($path)
    {
        // check whether the given parameter ($path) is a string,
        // throw an exception otherwise
        if (!is_string($path)) {
            throw new Exception('Path must be a string', 500);
        }

        // return whether path is excluded
        return in_array($path, $this->_excluded);
    }

    /**
     * Sets content.
     *
     * @param string $content
     * @throws Exception
     * @return \Sprockets\Context\Context
     */
    public function setContent($content)
    {
        // check whether the given parameter ($content) is a string,
        // throw an exception otherwise
        if (!is_string($content)) {
            throw new Exception('Content must be a string', 500);
        }

        // set content
        $this->_content = $content;

        // return self
        return $this;
    }

    /**
     * Returns content.
     *
     * @return string
     */
    public function getContent()
    {
        // return content
        return $this->_content;
    }
}
