<?php

namespace Sprockets\Directive;

/**
 * Directive class.
 *
 * @author Gabor Sar <gabor.sar@artoon.hu>
 * @package Sprockets\Directive
 */
class Directive
    implements \Sprockets\Directive\DirectiveInterface
{
    /**
     * This is the original comment in the current file, can be used
     * to replace it in the current file, to the content of this directive.
     *
     * @var string
     */
    protected $_comment = '';

    /**
     * Name of this directive.
     * Helps to decide witch action have to be invoked.
     *
     * @var string
     */
    protected $_name = '';

    /**
     * Type of the path of this directive.
     * Helps to decide what kind of path we
     * looking for (directory or file).
     *
     * @var boolean
     */
    protected $_isFile = false;

    /**
     * Original arguments content.
     *
     * @var string
     */
    protected $_arguments = '';

    /**
     * This flag helps to decide where to look for a path.
     * If this is true than the path is relative to the current file.
     * If this is false than the path is relative to one of our load paths.
     *
     * @var boolean
     */
    protected $_isRelativeToTheCurrentFile = false;

    /**
     * The path we are looking for.
     * Kept in filtered format, all useles characters are removed.
     *
     * @var string
     */
    protected $_path = '';

    /**
     * This array helps to decide whether this directive needs a file or
     * a directory as its path.
     *
     * @var array
     */
    protected static $_FILE_DIRECTIVE_NAMES = array(
        'require', 'include', 'stub', 'depend_on'
    );

    // regular expressions
    const PATH_FILTER_PATTERN = '/^\<?[\"\']?\.?(.*?)[\"\']?\>?\;?$/';
    const RELATIVE_TEST_PATTERN = '/^\<?[\"\']?\.\/.*?[\"\']?\>?\;?$/';

    /**
     * Sets comment.
     *
     * @param string $comment
     * @throws Exception
     * @return \Sprockets\Directive\Directive
     */
    public function setComment($comment)
    {
        // check whether the given parameter ($comment) is a string,
        // throw an exception otherwise
        if (!is_string($comment)) {
            throw new Exception('Comment must be a string', 500);
        }

        // set comment
        $this->_comment = $comment;

        // return self
        return $this;
    }

    /**
     * Sets name of directive.
     *
     * @param string $name
     * @throws Exception
     * @return \Sprockets\Directive\Directive
     */
    public function setName($name)
    {
        // check whether the given parameter ($name) is a string,
        // throw an exception otherwise
        if (!is_string($name)) {
            throw new Exception('Name must be a string', 500);
        }

        // set name
        $this->_name = $name;

        // check if this is a file-type directive
        $this->_updateIsFile();

        // return sels
        return $this;
    }

    /**
     * Sets arguments.
     *
     * @param string $arguments
     * @throws Exception
     * @return \Sprockets\Directive\Directive
     */
    public function setArguments($arguments)
    {
        // check whether the given parameter ($arguments) is a string,
        // throw an exception otherwise
        if (!is_string($arguments)) {
            throw new Exception('Arguments must be a string', 500);
        }

        // set arguments
        $this->_arguments = $arguments;

        // get filtered path information
        $this->_updatePath();

        // check if this path is relative to the current file
        $this->_updateIsRelativeToTheCurrentFile();

        // return self
        return $this;
    }

    /**
     * Returns the original comment.
     * Can be used to replace it in the current file,
     * to the content of this directive.
     *
     * @return string
     */
    public function getComment()
    {
        // return comment
        return $this->_comment;
    }

    /**
     * Returns name.
     * Can be used to decide witch action have to be invoked.
     *
     * @return string
     */
    public function getName()
    {
        // return name
        return $this->_name;
    }

    /**
     * Returns whether this is a file-type directive.
     *
     * @return boolean
     */
    public function isFile()
    {
        // return $_isFile flag
        return $this->_isFile;
    }

    /**
     * Returns the path of this directive.
     * Appends extension if required.
     *
     * @param string $extension Default value = js (javascript).
     * @throws Exception
     * @return string
     */
    public function getPath($extension = 'js')
    {
        // check whether the given parameter ($extension) is a string,
        // throw an exception otherwise
        if (!is_string($extension)) {
            throw new Exception('Extension must be a string', 500);
        }

        // return path
        return $this->_isFile
            ? $this->_path . '.' . $extension
            : $this->_path;
    }

    /**
     * Returns whether the path of this directive is relative to
     * the current file.
     *
     * @return boolean
     */
    public function isRelativeToTheCurrentFile()
    {
        // return $_isRelativeToTheCurrentFile flag
        return $this->_isRelativeToTheCurrentFile;
    }

    /**
     * Updates $_isFile attribute, using $_name to get it.
     */
    protected function _updateIsFile()
    {
        $this->_isFile = in_array($this->_name, self::$_FILE_DIRECTIVE_NAMES);
    }

    /**
     * Updates $_path attribute, using $_arguments to get it.
     */
    protected function _updatePath()
    {
        $this->_path = preg_replace(
            self::PATH_FILTER_PATTERN,
            '$1',
            $this->_arguments
        );
    }

    /**
     * Updates $_isRelativeToTheCurrentFile attribute, using $_arguments
     * to get it.
     */
    protected function _updateIsRelativeToTheCurrentFile()
    {
        $this->_isRelativeToTheCurrentFile = (bool) preg_match(
            self::RELATIVE_TEST_PATTERN,
            $this->_arguments
        );
    }
}
